﻿using Kivii.DataAnnotations;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Entities
{
    [Api(Description = "账单")]
    [Alias(Configs.TableNameBill)]
    public class Bill : EntityWithMetadata, IEntityInAssemblyDb,
        IEntityHasSummary, IEntityHasRemark,
        IEntityHasCreator, IEntityHasUpdater
    {

        #region 关联信息 Owner
        /// <summary>
        /// 一般不使用此字段，预留
        /// </summary>
        [DefaultEmptyGuid]
        public Guid OwnerKvid { get; set; }

        [StringLength(500)]
        public string OwnerName { get; set; }

        [InternalSetter]
        [DefaultEmptyGuid]
        public Guid BizKvid { get; set; }

        [ApiMember(Description = "业务流水号")]
        [InternalSetter]
        [StringLength(200)]
        public string BizId { get; set; }

        [ApiMember(Description = "业务类型")]
        [InternalSetter]
        [StringLength(100)]
        public string BizType { get; set; }

        #endregion

        #region 账单双方 Payer需方单位  Payee供方单位
        [IgnoreUpdate]
        [DefaultEmptyGuid, Required]
        public Guid PayerKvid { get; set; }

        [ApiMember(Description = "付款单位")]
        [IgnoreUpdate]
        [StringLength(100), Required]
        public string PayerName { get; set; }

        [IgnoreUpdate]
        [DefaultEmptyGuid, Required]
        public Guid PayeeKvid { get; set; }

        [ApiMember(Description = "收款单位")]
        [IgnoreUpdate]
        [StringLength(100), Required]
        public string PayeeName { get; set; }
        #endregion

        #region 账单双方联系方式
        [DefaultEmptyGuid]
        public Guid PayerContactKvid { get; set; }

        [ApiMember(Description = "付款联系人")]
        [StringLength(100), Default("")]
        public string PayerContactName { get; set; }

        [ApiMember(Description = "付款联系电话")]
        [StringLength(50), Default("")]
        public string PayerContactNumber { get; set; }

        [ApiMember(Description = "付款联系邮箱")]
        [StringLength(100), Default("")]
        public string PayerContactEmail { get; set; }

        [ApiMember(Description = "付款联系地址")]
        [StringLength(500), Default("")]
        public string PayerContactAddress { get; set; }

        [DefaultEmptyGuid]
        public Guid PayeeContactKvid { get; set; }

        [ApiMember(Description = "收款联系人")]
        [StringLength(100), Default("")]
        public string PayeeContactName { get; set; }

        [ApiMember(Description = "收款联系电话")]
        [StringLength(50), Default("")]
        public string PayeeContactNumber { get; set; }

        [ApiMember(Description = "收款联系邮箱")]
        [StringLength(100), Default("")]
        public string PayeeContactEmail { get; set; }

        [ApiMember(Description = "收款联系地址")]
        [StringLength(500), Default("")]
        public string PayeeContactAddress { get; set; }
        #endregion

        #region 基本信息
        [ApiMember(Description = "流水号")]
        [IgnoreUpdate]
        [StringLength(100), Default("")]
        public string SerialNumber { get; set; }
        #endregion

        #region 金额相关
        [ApiMember(Description = "预结算金额")]
        [IgnoreUpdate]
        [DecimalLength(15, 2), Default(0)]
        public decimal AmountPlan { get; set; }

        [ApiMember(Description = "结算金额,实际结算金额")]
        [InternalSetter]
        [DecimalLength(15, 2), Default(0)]
        public decimal Amount { get; set; }

        [ApiMember(Description = "已付金额")]
        [InternalSetter]
        [DecimalLength(15, 2), Default(0)]
        public decimal AmountPayment { get; set; }

        [ApiMember(Description = "单位")]
        [IgnoreUpdate]
        [StringLength(50), Required]
        public CurrencyUnit Currency { get; set; }
        #endregion

        #region 文本 摘要 备注
        [ApiMember(Description = "摘要，最大500字")]
        [StringLength(500), Default("")]
        public string Summary { get; set; }

        [ApiMember(Description = "备注，最大2000字")]
        [StringLength(2000), Default("")]
        public string Remark { get; set; }
        #endregion

        #region 操作相关  Creator Updater Operator OperateTime
        [ApiMember(Description = "操作日期")]
        [IgnoreUpdate]
        [Required]
        public DateTime OperateTime { get; set; }

        [IgnoreUpdate]
        [DefaultEmptyGuid]
        public Guid OperatorKvid { get; set; }

        [ApiMember(Description = "经 办 人")]
        [IgnoreUpdate]
        [StringLength(100), Default("")]
        public string OperatorName { get; set; }

        [ApiMember(Description = "组织机构名称,当组织有子组织时区分所属组织")]
        [IgnoreUpdate]
        [StringLength(200), CurrentOrganizationName]
        public string OrganizationName { get; set; }

        [ApiMember(Description = "组织机构Id,当组织有子组织时区分所属组织")]
        [IgnoreUpdate]
        [CurrentOrganizationKvid]
        public Guid OrganizationKvid { get; set; }

        [ApiMember(Description = "创建人Kvid ")]
        [IgnoreUpdate]
        [CurrentMemberKvid]
        public Guid CreatorKvid { get; set; }

        [ApiMember(Description = "创建人")]
        [IgnoreUpdate]
        [StringLength(50), CurrentMemberName]
        public string CreatorName { get; set; }

        [ApiMember(Description = "更新人Kvid ")]
        [CurrentMemberKvid]
        public Guid UpdaterKvid { get; set; }

        [ApiMember(Description = "更新人")]
        [StringLength(50), CurrentMemberName]
        public string UpdaterName { get; set; }
        #endregion
    }

    public class BillWithSettlements:Bill
    {
        [Ignore]
        public List<Settlement> Settlements { get; set; }

    }
}
