﻿using Kivii.DataAnnotations;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Entities
{
    /// <summary>
    /// Status:1业务员借票申请,25申请通过工作流待财务审批,50财务通过申请,51待开票,52正在开票中 55,56开票失败 57驳回 maxvalue开票完成
    /// 当多条借票申请对应收款时 生成一个总和的申请与收款关联 更新对应的发票的ApplyKvid为总和的Apply的Kvid  原有的申请Apply的ParentKvid记录总和Apply的Kvid
    /// </summary>
    [Api(Description = "发票申请")]
    [Alias(Configs.TableNameApply)]
    public class InvoiceApply : EntityWithMetadata, IEntityInAssemblyDb,
        IEntityHasSummary, IEntityHasRemark,
        IEntityHasOffset, IEntityHasBiz,
        IEntityHasCreator, IEntityHasUpdater, IEntityHasOrganization
    {
        #region 关联信息 Owner,Offset Biz,全内部设置
        /// <summary>
        /// 一般不使用此字段，预留
        /// </summary>
        [InternalSetter]
        [DefaultEmptyGuid]
        public Guid OwnerKvid { get; set; }

        [InternalSetter]
        [StringLength(500)]
        public string OwnerName { get; set; }
        /// <summary>
        /// 冲帐关联
        /// </summary>
        [InternalSetter]
        [DefaultEmptyGuid]
        public Guid OffsetKvid { get; set; }

        [InternalSetter]
        [DefaultEmptyGuid]
        public Guid BizKvid { get; set; }

        [ApiMember(Description = "业务流水号")]
        [InternalSetter]
        [StringLength(200)]
        public string BizId { get; set; }

        [ApiMember(Description = "业务类型")]
        [InternalSetter]
        [StringLength(100)]
        public string BizType { get; set; }

        [InternalSetter]
        [DefaultEmptyGuid]
        public Guid RootKvid { get; set; }

        [InternalSetter]
        [DefaultEmptyGuid]
        public Guid ParentKvid { get; set; }
        #endregion

        #region 申请信息

        [ApiMember(Description = "申请编号，唯一性")]
        [InternalSetter, Unique]
        [StringLength(100), Default(""),SerialNumber]
        public string SerialNumber { get; set; }

        [ApiMember(Description ="申请类型")]
        [IgnoreUpdate]
        [StringLength(50),Required]
        public InvoiceApplyType OperateType { get; set; }

        [IgnoreUpdate]
        [DefaultEmptyGuid]
        public Guid PayerKvid { get; set; }

        [ApiMember(Description = "付款单位,贷方")]
        [IgnoreUpdate]
        [StringLength(100), Required]
        public string PayerName { get; set; }

        [ApiMember(Description = "付方税号,收票方税号")]
        [IgnoreUpdate]
        [StringLength(100), Required]
        public string PayerTaxNumber { get; set; }

        [ApiMember(Description = "公司地址")]
        [StringLength(200), Default("")]
        public string PayerCompanyAddress { get; set; }

        [ApiMember(Description = "电话号码")]
        [StringLength(50), Default("")]
        public string PayerPhone { get; set; }

        [ApiMember(Description = "付款账户名称,指开户银行")]
        [IgnoreUpdate]
        [StringLength(200), Default("")]
        public string PayerRegisteredBank { get; set; }

        [ApiMember(Description = "付款账户卡号")]
        [IgnoreUpdate]
        [StringLength(100), Default("")]
        public string PayerBankAccount { get; set; }

        [IgnoreUpdate]
        [DefaultEmptyGuid]
        public Guid PayeeKvid { get; set; }

        [ApiMember(Description = "收款单位,借方")]
        [IgnoreUpdate]
        [StringLength(100), Required]
        public string PayeeName { get; set; }

        [ApiMember(Description = "收方税号,开票方税号")]
        [IgnoreUpdate]
        [StringLength(100), Required]
        public string PayeeTaxNumber { get; set; }

        [ApiMember(Description = "公司地址")]
        [IgnoreUpdate]
        [StringLength(200), Default("")]
        public string PayeeCompanyAddress { get; set; }

        [ApiMember(Description = "电话号码")]
        [IgnoreUpdate]
        [StringLength(50), Default("")]
        public string PayeePhone { get; set; }

        [ApiMember(Description = "收款账户名称,指开户银行")]
        [IgnoreUpdate]
        [StringLength(200), Default("")]
        public string PayeeRegisteredBank { get; set; }

        [ApiMember(Description = "收款账户卡号")]
        [IgnoreUpdate]
        [StringLength(100), Default("")]
        public string PayeeBankAccount { get; set; }
        #endregion

        #region 申请金额
        [ApiMember(Description = "发票类型，普通，专用增值等")]
        [InternalSetter]
        [StringLength(50)]
        public string Type { get; set; }

        [ApiMember(Description = "申请金额")]
        [InternalSetter]
        [DecimalLength(15, 2), Default(0)]
        public decimal Amount { get; set; }

        [ApiMember(Description = "已开票金额")]
        [InternalSetter]
        [DecimalLength(15, 2), Default(0)]
        public decimal AmountInvoice { get; set; }

        [ApiMember(Description = "已关联金额,关联的收款或者刷卡现金账户金额")]
        [InternalSetter]
        [DecimalLength(15, 2), Default(0)]
        public decimal AmountUsed { get; set; }

        [ApiMember(Description = "单位")]
        [IgnoreUpdate]
        [StringLength(50), Required]
        public CurrencyUnit Currency { get; set; }
        #endregion

        #region 操作相关  Organization Creator Updater Operator OperateTime 全禁止更新

        [ApiMember(Description = "申请日期")]
        [IgnoreUpdate]
        [Required]
        public DateTime OperateTime { get; set; }

        [IgnoreUpdate]
        [DefaultEmptyGuid]
        public Guid OperatorKvid { get; set; }

        [ApiMember(Description = "申请人")]
        [IgnoreUpdate]
        [StringLength(100)]
        public string OperatorName { get; set; }


        [ApiMember(Description = "组织机构Id,当组织有子组织时区分所属组织")]
        [IgnoreUpdate]
        [CurrentOrganizationKvid]
        public Guid OrganizationKvid { get; set; }

        [ApiMember(Description = "创建人Kvid ")]
        [IgnoreUpdate]
        [CurrentMemberKvid]
        public Guid CreatorKvid { get; set; }

        [ApiMember(Description = "创建人")]
        [IgnoreUpdate]
        [StringLength(50), CurrentMemberName]
        public string CreatorName { get; set; }

        [ApiMember(Description = "更新人Kvid ")]
        [CurrentMemberKvid]
        public Guid UpdaterKvid { get; set; }

        [ApiMember(Description = "更新人")]
        [StringLength(50), CurrentMemberName]
        public string UpdaterName { get; set; }

        #endregion

        #region 文本 摘要 备注
        [ApiMember(Description = "摘要，最大500字")]
        [StringLength(500), Default("")]
        public string Summary { get; set; }

        [ApiMember(Description = "备注，最大2000字")]
        [StringLength(2000), Default("")]
        public string Remark { get; set; }
        #endregion
    }

    public enum InvoiceApplyType
    {
        Unsupported = 0,
        /// <summary>
        /// 收款
        /// </summary>
        Payment,
        /// <summary>
        /// 业务
        /// </summary>
        Biz,
        /// <summary>
        /// 借票
        /// </summary>
        Debit,
        /// <summary>
        /// 关联,与收款一一对应用
        /// </summary>
        Related
    }

    public enum InvoiceApplyStatus
    {
        /// <summary>
        /// 默认状态
        /// </summary>
        Default = 0,
        /// <summary>
        /// 借票申请初始状态
        /// </summary>
        Debit = 1,
        /// <summary>
        /// 流程通过
        /// </summary>
        ProcessAdoption = 25,
        /// <summary>
        /// 财务审批通过
        /// </summary>
        FinancialApproval = 50,
        /// <summary>
        /// 财务执行开票动作
        /// </summary>
        FinancialExecute = 51,
        /// <summary>
        /// 进入自动开票流程
        /// </summary>
        AutoExecuting = 52,
        /// <summary>
        /// 自动开票生产XML文件失败
        /// </summary>
        ErrorXml = 55,
        /// <summary>
        /// 插入发票到数据库失败
        /// </summary>
        ErrorInsertInvoice = 56,
        /// <summary>
        /// 申请驳回
        /// </summary>
        ApplyReject = 57,
        /// <summary>
        /// 发票作废
        /// </summary>
        InvoiceOffset = 58,
        /// <summary>
        /// 申请合并
        /// </summary>
        ApplyMerge = 59,
        /// <summary>
        /// 红字冲销
        /// </summary>
        InvoiceNegative = 60,
        /// <summary>
        /// 申请完成,已开具发票
        /// </summary>
        Completed = int.MaxValue
    }
}
