﻿using Kivii.DataAnnotations;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Entities
{
    [Api(Description = "付款")]
    [Alias(Configs.TableNamePayment)]
    public class Payment : EntityWithMetadata,IEntityInAssemblyDb,
        IEntityHasSummary, IEntityHasRemark,
        IEntityHasOffset, IEntityHasBiz,
        IEntityHasCreator, IEntityHasUpdater, IEntityHasOrganization
    {
        #region 关联信息 Owner,Offset Biz
        /// <summary>
        /// 一般不使用此字段，预留
        /// </summary>
        [ApiMember(Description = "所属Kvid,一般用于这笔付款属于哪个部门")]
        [InternalSetter]
        [DefaultEmptyGuid]
        public Guid OwnerKvid { get; set; }

        [InternalSetter]
        [StringLength(500)]
        public string OwnerName { get; set; }
        /// <summary>
        /// 冲帐关联
        /// </summary>
        [InternalSetter]
        [DefaultEmptyGuid]
        public Guid OffsetKvid { get; set; }

        [InternalSetter]
        [DefaultEmptyGuid]
        public Guid BizKvid { get; set; }

        [ApiMember(Description = "业务流水号")]
        [InternalSetter]
        [StringLength(200)]
        public string BizId { get; set; }

        [ApiMember(Description = "业务类型")]
        [InternalSetter]
        [StringLength(100)]
        public string BizType { get; set; }

        /// <summary>
        /// 根kvid,原始的付款信息，所以拆分的N级子项都有共同的RootKvid
        /// </summary>
        [InternalSetter]
        [DefaultEmptyGuid]
        public Guid RootKvid { get; set; }

        //ParentKvid为 Empty为主发票，否则为拆分收款，不作统计用
        [InternalSetter]
        [DefaultEmptyGuid]
        public Guid ParentKvid { get; set; }

        #endregion

        #region 收付双方 Payer付款单位,贷方  Payee收款单位,借方,账号相关
        [IgnoreUpdate]
        [DefaultEmptyGuid, Required]
        public Guid PayerKvid { get; set; }

        [ApiMember(Description = "付款单位,贷方")]
        [IgnoreUpdate]
        [StringLength(100), Required]
        public string PayerName { get; set; }

        [IgnoreUpdate]
        [DefaultEmptyGuid]
        public Guid PayerAccountKvid { get; set; }


        [ApiMember(Description = "付款账户")]
        [IgnoreUpdate]
        [StringLength(200), Default("")]
        public string PayerAccountOwnerName { get; set; }

        [ApiMember(Description = "付款账户名称,如果是Bank类的，指银行")]
        [IgnoreUpdate]
        [StringLength(200), Default("")]
        public string PayerAccountName { get; set; }

        [IgnoreUpdate]
        [StringLength(100), Default("")]
        public string PayerAccountSerialNumber { get; set; }

        [IgnoreUpdate]
        [DefaultEmptyGuid, Required]
        public Guid PayeeKvid { get; set; }

        [ApiMember(Description = "收款单位,借方")]
        [IgnoreUpdate]
        [StringLength(100), Required]
        public string PayeeName { get; set; }

        [IgnoreUpdate]
        [DefaultEmptyGuid]
        public Guid PayeeAccountKvid { get; set; }

        [ApiMember(Description = "收款账户")]
        [IgnoreUpdate]
        [StringLength(200), Default("")]
        public string PayeeAccountOwnerName { get; set; }

        [ApiMember(Description = "收款账户名称")]
        [IgnoreUpdate]
        [StringLength(200), Default("")]
        public string PayeeAccountName { get; set; }

        [ApiMember(Description = "收款账户卡号")]
        [IgnoreUpdate]
        [StringLength(100), Default("")]
        public string PayeeAccountSerialNumber { get; set; }
        #endregion

        #region 收付款相关
        [ApiMember(Description = "流水号")]
        [IgnoreUpdate]
        [StringLength(100), Default(""), SerialNumber]
        public string SerialNumber { get; set; }

        [ApiMember(Description = "付款类型,现金、银行汇款等")]
        [IgnoreUpdate]
        [StringLength(100), Required]
        public PaymentType Type { get; set; }

        [ApiMember(Description = "发生金额")]
        [InternalSetter]
        [DecimalLength(15, 2), Default(0)]
        public decimal Amount { get; set; }

        [ApiMember(Description = "已用金额，被业务关联掉的金额")]
        [InternalSetter]
        [DecimalLength(15, 2), Default(0)]
        public decimal AmountUsed { get; set; }

        //如果被拆分后，这个金额应该是拆分项目被关联金额的统计
        [ApiMember(Description = "拆分金额，被拆分掉的金额")]
        [InternalSetter]
        [DecimalLength(15, 2), Default(0)]
        public decimal AmountSplited { get; set; }

        [ApiMember(Description = "开票金额")]
        [InternalSetter]
        [DecimalLength(15, 2), Default(0)]
        public decimal AmountInvoice { get; set; }

        [ApiMember(Description = "单位")]
        [IgnoreUpdate]
        [StringLength(50), Required]
        public CurrencyUnit Currency { get; set; }

        #endregion

        #region 文本 摘要 备注
        [ApiMember(Description = "摘要，最大500字")]
        [IgnoreUpdate]
        [StringLength(500), Default("")]
        public string Summary { get; set; }

        [ApiMember(Description = "备注，最大2000字")]
        [StringLength(2000), Default("")]
        public string Remark { get; set; }
        #endregion

        #region 操作相关  Organization Creator Updater Operator OperateTime 

        [ApiMember(Description = "操作日期")]
        [IgnoreUpdate]
        [Required]
        public DateTime OperateTime { get; set; }

        [IgnoreUpdate]
        [DefaultEmptyGuid]
        public Guid OperatorKvid { get; set; }

        [ApiMember(Description = "经办人")]
        [IgnoreUpdate]
        [StringLength(100), Default("")]
        public string OperatorName { get; set; }

        [IgnoreUpdate]
        [DefaultEmptyGuid]
        public Guid AuditorKvid { get; set; }

        [ApiMember(Description ="审核人")]
        [IgnoreUpdate]
        [StringLength(100),Default("")]
        public string AuditorName { get; set; }


        [ApiMember(Description ="组织机构Id,当组织有子组织时区分所属组织")]
        [IgnoreUpdate]
        public Guid OrganizationKvid { get; set; }

        [ApiMember(Description = "创建人Kvid ")]
        [IgnoreUpdate]
        [CurrentMemberKvid]
        public Guid CreatorKvid { get; set; }

        [ApiMember(Description = "创建人")]
        [IgnoreUpdate]
        [StringLength(50), CurrentMemberName]
        public string CreatorName { get; set; }

        [ApiMember(Description = "更新人Kvid ")]
        [CurrentMemberKvid]
        public Guid UpdaterKvid { get; set; }

        [ApiMember(Description = "更新人")]
        [StringLength(50), CurrentMemberName]
        public string UpdaterName { get; set; }

        #endregion

    }

    public enum PaymentType
    {
        Unsupported = 0,

        /// <summary>
        /// 银行流水
        /// </summary>
        Bank,

        /// <summary>
        /// 内部拆分,Biz业务认领使用类型
        /// </summary>
        Split,

        /// <summary>
        /// 非业务使用
        /// </summary>
        UnBiz,

        /// <summary>
        /// 退款
        /// </summary>
        Refund,

        /// <summary>
        /// 现金,直接从现金内部账户拨款到Payment的类型(区别于Account类型[消费使用],开票申请使用)
        /// </summary>
        Cash,

        /// <summary>
        /// 支付宝支付,暂无使用
        /// </summary>
        AliPay,

        /// <summary>
        /// 微信支付,暂无使用
        /// </summary>
        WeChat,

        /// <summary>
        /// 动态拆分,必有Biz,即为用于关联支付Pay业务,子集Pay用于结算,当有Pay冲销则此状态Amount AmountUsed AmountSplited都发生变化
        /// </summary>
        DynamicSplit,

        /// <summary>
        /// 刷卡,直接从刷卡内部账户拨款到Payment的类型(区别于Account类型[消费使用],开票申请使用)
        /// </summary>
        Pos,

        /// <summary>
        /// 支付,此类型必有Biz,即为动态拆分的子集用于关联具体Settlement业务
        /// </summary>
        Pay,

        /// <summary>
        /// 转账,必有Biz
        /// </summary>
        Transfer
    }
}
