﻿using Kivii.Finances.Entities;
using Kivii.Linq;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances
{
    public static class PayExtension
    {
        /// <summary>
        /// 使用单笔到账 付款登记
        /// </summary>
        /// <param name="pay"></param>
        /// <param name="payment"></param>
        /// <param name="payerAccount"></param>
        /// <param name="payeeAccount"></param>
        /// <returns></returns>
        public static Payment Paying(this Pay pay, Payment payment, Account payerAccount, Account payeeAccount, decimal amount = 0)
        {
            pay.ThrowIfNull("未传入付款登记信息!");
            (pay.AmountPayment >= pay.AmountSettlement).ThrowIfTrue("已完成付款登记,无需重复登记!");
            payment.ThrowIfNull("未传入到账付款信息!");
            payerAccount.ThrowIfNull("请传入付款账户!");
            payeeAccount.ThrowIfNull("请传入收款账户!");
            (payment.Currency != pay.Currency).ThrowIfTrue("存在不相符的货币单位!");
            if (payment.Type != PaymentType.Split && payment.Type != PaymentType.Pos && payment.Type != PaymentType.Cash && payment.Type != PaymentType.AliPay && payment.Type != PaymentType.WeChat) throw new Exception("请选择正确的付款方式!");

            if (amount <= 0 || amount > pay.AmountSettlement - pay.AmountPayment) amount = pay.AmountSettlement - pay.AmountPayment;//付款单所需付款的总金额
            var amountCanUsed = payment.Amount - payment.AmountUsed;//到账可使用金额
            (amountCanUsed < amount).ThrowIfTrue($"付款编号[{payment.SerialNumber}]余额不足!可用余额:{amountCanUsed}");

            pay.AmountPayment += amount;
            pay.Amount = pay.AmountPayment - pay.AmountDiscount;
            pay.PayedTime = payment.OperateTime;

            var dynamicSplitPaymentKvid = Guid.NewGuid();
            #region 生成DynamicSplit类型的子payment(Biz记录Pay),调用payment的Accept方法 同时对收付账户进行变化
            var rtns = new Payment();
            rtns.OwnerKvid = payment.OwnerKvid;
            rtns.OwnerName = payment.OwnerName;
            rtns.Kvid = dynamicSplitPaymentKvid;
            rtns.RootKvid = payment.RootKvid;
            rtns.ParentKvid = payment.Kvid;

            rtns.BizId = pay.SerialNumber;
            rtns.BizKvid = pay.Kvid;
            rtns.BizType = typeof(Pay).FullName;

            rtns.Type = PaymentType.DynamicSplit;

            //payment直接用于消费 PayerAccount记录Balance
            rtns.PayerKvid = payment.PayerKvid;
            rtns.PayerName = payment.PayerName;
            rtns.PayerAccountKvid = payerAccount.Kvid;
            rtns.PayerAccountName = payerAccount.Name;
            rtns.PayerAccountOwnerName = payerAccount.OwnerName;
            rtns.PayerAccountSerialNumber = payerAccount.SerialNumber;

            //Payee为系统内部的默认业务账户收款
            rtns.PayeeKvid = payment.PayeeKvid;
            rtns.PayeeName = payment.PayeeName;
            rtns.PayeeAccountKvid = payeeAccount.Kvid;
            rtns.PayeeAccountName = payeeAccount.Name;
            rtns.PayeeAccountOwnerName = payeeAccount.OwnerName;
            rtns.PayeeAccountSerialNumber = payeeAccount.SerialNumber;

            rtns.Amount = amount;
            rtns.AmountUsed = amount;
            rtns.AmountSplited = amount;//已消费了,就不能再拆分了 所以已拆分金额要填充上

            rtns.OperateTime = DateTime.Now;
            rtns.OperatorKvid = KiviiContext.CurrentMember.Kvid;
            rtns.OperatorName = KiviiContext.CurrentMember.FullName;
            rtns.Currency = pay.Currency;
            #endregion

            return rtns;
        }

        /// <summary>
        /// 使用单个账户 付款登记
        /// </summary>
        /// <param name="pay"></param>
        /// <param name="payerAccount"></param>
        /// <param name="payeeAccount"></param>
        /// <returns></returns>
        public static Payment Paying(this Pay pay, Account payerAccount, Account payeeAccount, decimal amount = 0)
        {
            pay.ThrowIfNull("未传入付款登记信息!");
            (pay.AmountPayment >= pay.AmountSettlement).ThrowIfTrue("已完成付款登记,无需重复登记!");
            payerAccount.ThrowIfNull("请传入付款账户!");
            payeeAccount.ThrowIfNull("请传入收款账户!");
            (payerAccount.Currency != pay.Currency).ThrowIfTrue("存在不相符的货币单位!");
            (payeeAccount.Currency != pay.Currency).ThrowIfTrue("存在不相符的货币单位!");
            (payerAccount.Type != AccountType.Deposit && payerAccount.Type != AccountType.Discount).ThrowIfTrue("请选择正确的付款账户!");

            if (amount <= 0 || amount > pay.AmountSettlement - pay.AmountPayment) amount = pay.AmountSettlement - pay.AmountPayment;//付款单所需付款的总金额
            var amountCanUsed = payerAccount.Amount + payerAccount.CreditLine;//账户可使用金额
            (amountCanUsed < amount).ThrowIfTrue($"账户[{payerAccount.Name}]余额不足!可用余额:{amountCanUsed}");

            pay.AmountPayment += amount;
            pay.Amount = pay.AmountPayment - pay.AmountDiscount;
            pay.PayedTime = DateTime.Now;

            var dynamicSplitPaymentKvid = Guid.NewGuid();
            #region 生成DynamicSplit类型的子payment(Biz记录Pay),调用payment的Accept方法 同时对收付账户进行变化
            var rtns = new Payment();
            rtns.OwnerKvid = KiviiContext.CurrentMember.DepartmentKvid;
            rtns.OwnerName = KiviiContext.CurrentMember.DepartmentName;
            rtns.Kvid = dynamicSplitPaymentKvid;
            rtns.RootKvid = dynamicSplitPaymentKvid;
            rtns.ParentKvid = Guid.Empty;

            rtns.BizId = pay.SerialNumber;
            rtns.BizKvid = pay.Kvid;
            rtns.BizType = typeof(Pay).FullName;

            rtns.Type = PaymentType.DynamicSplit;

            //payment直接用于消费 PayerAccount记录Balance
            rtns.PayerKvid = pay.PayerKvid;
            rtns.PayerName = pay.PayerName;
            rtns.PayerAccountKvid = payerAccount.Kvid;
            rtns.PayerAccountName = payerAccount.Name;
            rtns.PayerAccountOwnerName = payerAccount.OwnerName;
            rtns.PayerAccountSerialNumber = payerAccount.SerialNumber;

            //Payee为系统内部的默认业务账户收款
            rtns.PayeeKvid = pay.PayeeKvid;
            rtns.PayeeName = pay.PayeeName;
            rtns.PayeeAccountKvid = payeeAccount.Kvid;
            rtns.PayeeAccountName = payeeAccount.Name;
            rtns.PayeeAccountOwnerName = payeeAccount.OwnerName;
            rtns.PayeeAccountSerialNumber = payeeAccount.SerialNumber;

            rtns.Amount = amount;
            rtns.AmountUsed = amount;
            rtns.AmountSplited = amount;//已消费了,就不能再拆分了 所以已拆分金额要填充上

            rtns.OperateTime = DateTime.Now;
            rtns.OperatorKvid = KiviiContext.CurrentMember.Kvid;
            rtns.OperatorName = KiviiContext.CurrentMember.FullName;
            rtns.Currency = pay.Currency;
            #endregion

            return rtns;
        }

        /// <summary>
        /// 付款登记折扣金额记录
        /// </summary>
        /// <param name="pay"></param>
        /// <param name="discountAccount"></param>
        /// <param name="amount"></param>
        /// <param name="remark"></param>
        /// <returns></returns>
        public static Discount Discount(this Pay pay, Account discountAccount, decimal amount, string remark)
        {
            pay.ThrowIfNull("未传入付款登记信息!");
            discountAccount.ThrowIfNull("请传入收款账户!");
            (discountAccount.Type != AccountType.Discount).ThrowIfTrue("请选择正确的折扣账户!");
            (discountAccount.Currency != pay.Currency).ThrowIfTrue("存在不相符的货币单位!");
            (amount <= 0).ThrowIfTrue("抹零抵扣金额不能少于0!");
            (amount > pay.AmountSettlement - pay.AmountPayment).ThrowIfTrue("折扣金额超过付款登记金额!");

            pay.AmountPayment += amount;
            pay.AmountDiscount = amount;
            pay.Amount = pay.AmountPayment - pay.AmountDiscount;
            pay.Remark = remark;

            #region 记录折扣抹零操作
            var rtns = new Discount();
            rtns.OwnerKvid = KiviiContext.CurrentMember.DepartmentKvid;
            rtns.OwnerName = KiviiContext.CurrentMember.DepartmentName;
            rtns.PayKvid = pay.Kvid;
            rtns.AccountKvid = discountAccount.Kvid;
            rtns.AccountName = discountAccount.Name;
            rtns.AccountSerialNumber = discountAccount.SerialNumber;
            rtns.Amount = amount;
            rtns.Currency = pay.Currency;
            rtns.OperateTime = DateTime.Now;
            rtns.OperatorKvid = KiviiContext.CurrentMember.Kvid;
            rtns.OperatorName = KiviiContext.CurrentMember.FullName;
            rtns.Summary = $"{rtns.OperatorName}于{rtns.OperateTime} 对{pay.PayerName}[{pay.SerialNumber}]进行抹零折扣操作,金额为:{amount} {pay.Currency}";
            rtns.Remark = remark;
            #endregion

            return rtns;
        }

        /// <summary>
        /// 作废
        /// </summary>
        /// <param name="pay"></param>
        /// <param name="remark"></param>
        /// <returns></returns>
        public static Pay Offset(this Pay pay, string remark)
        {
            pay.ThrowIfNull("付款记录不能为空!");
            (pay.Currency == CurrencyUnit.Unsupported).ThrowIfTrue("不支持的货币单位");
            (pay.OffsetKvid != Guid.Empty).ThrowIfTrue("收项已冲账");

            #region payment冲账
            var newPayKvid = Guid.NewGuid();
            pay.OffsetKvid = newPayKvid;
            pay.AddOnlyProperties(o => o.OffsetKvid);
            pay.Remark = $"{remark}";
            pay.AddOnlyProperties(o => o.Remark);

            //新建一个和原来一样的
            var offsetPay = new Pay();
            offsetPay.PopulateWith(pay);
            offsetPay.OffsetKvid = pay.Kvid;//新的冲帐关联用原来的Kvid
            offsetPay.Kvid = newPayKvid;
            offsetPay.SerialNumber = pay.SerialNumber.IsNullOrEmpty() ? "" : ("-" + pay.SerialNumber);
            offsetPay.Remark = $"{remark}";
            offsetPay.AmountDiscount = 0 - offsetPay.AmountDiscount;
            offsetPay.AmountPayment = 0 - offsetPay.AmountPayment;
            offsetPay.AmountSettlement = 0 - offsetPay.AmountSettlement;
            offsetPay.Amount = 0 - offsetPay.Amount;//反向值
            #endregion

            return offsetPay;
        }
    }
}
