﻿using Kivii.Finances.Entities;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Transforms
{
    [Api(Description = "付款登记")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PayPaying : RestfulExecution<Pay>
    {
        public List<Guid> SettlementKvids { get; set; }
        public List<PayingMethod> PayingMethods { get; set; }//付款方式,支持组合付款
        public Pay Item { get; set; }

        private static MethodInfo _realMethod = null;
        public override object OnExecution(IRequest req, IResponse res)
        {
            SettlementKvids.ThrowIfNullOrEmpty("未传入结算信息,无法付款登记!");
            Item.ThrowIfNull("未传入付款登记信息!");
            Item.PayerName.ThrowIfNullOrEmpty("未传入付款单位!");
            Item.PayeeName.ThrowIfNullOrEmpty("未传入收款单位!");
            Item.Currency.ThrowIfNull("货币单位不能为空!");
            PayingMethods.ThrowIfNullOrEmpty("未传入付款登记方式!");
            PayingMethods.Exists(o => o.Type == PayType.Unsupported).ThrowIfTrue("存在不支持的付款登记方式!");
            PayingMethods.Exists(o => o.Kvid == Guid.Empty).ThrowIfTrue("付款登记Kvid不能为空!");
            (PayingMethods.Count(o => o.Type == PayType.Discount) > 1).ThrowIfTrue("金额抵扣登记方式超过限制次数,仅可使用一次");
            var amountMethod = PayingMethods.Sum(o => o.Amount);

            var conn = KiviiContext.GetOpenedDbConnection<Pay>();
            var settlements = conn.Select<Settlement>(o => Sql.In(o.Kvid, SettlementKvids));
            settlements.ThrowIfNullOrEmpty("未找到结算单!");
            if (settlements.Count != SettlementKvids.Count) throw new Exception("所选结算项与实际结算项不符合!");
            if (settlements.Exists(o => o.OffsetKvid != Guid.Empty)) throw new Exception("所选结算项存在已作废项!");
            if (settlements.Exists(o => o.PayKvid != Guid.Empty)) throw new Exception("所选结算项存在已付款登记!");
            var amountSettlement = settlements.Sum(o => o.Amount);
            var amountSettlementPlan = settlements.Sum(o => o.AmountPlan);
            (amountMethod != amountSettlement).ThrowIfTrue("所选结算总金额与付款总金额不等!");

            var rtns = new RestfulExecutionResponse<Pay>();
            rtns.Results = new List<Pay>();

            var payKvid = Guid.NewGuid();
            #region 创建支付单
            Item.OwnerKvid = KiviiContext.CurrentMember.DepartmentKvid;
            Item.OwnerName = KiviiContext.CurrentMember.DepartmentName;
            Item.OperateTime = DateTime.Now;
            Item.OperatorKvid = KiviiContext.CurrentMember.Kvid;
            Item.OperatorName = KiviiContext.CurrentMember.FullName;
            Item.Kvid = payKvid;
            Item.SerialNumber = string.Empty;
            Item.AmountSettlement = amountSettlement;
            Item.Amount = 0;
            Item.AmountDiscount = 0;
            Item.AmountPayment = 0;
            #endregion
            foreach (var item in settlements)//更新PayKvid
            {
                item.PayKvid = payKvid;
                item.AddOnlyProperties(o => o.PayKvid);
            }
            var accountBiz = Item.Currency.GetBizAccount();//conn.Single<Account>(o => o.OwnerKvid == KiviiContext.CurrentMember.OrganizationKvid && o.Type == AccountType.Biz && o.Currency == Item.Currency);
            accountBiz.ThrowIfNull($"未能正确获取系统{Item.Currency}业务账户!请联系管理员!");
            var accountBalance = accountBiz.Currency.GetBalanceAccount();
            accountBalance.ThrowIfNull($"未能正确获取系统{accountBiz.Currency}平衡账户!请联系管理员!");

            if (PayingMethods.Count == 1)//单笔付款登记
            {
                var payingMethod = PayingMethods[0];
                if (payingMethod.Type == PayType.Payment)
                {
                    var payment = conn.SingleById<Payment>(payingMethod.Kvid);
                    payment.ThrowIfNull("所选付款数据不存在!");
                    (payment.Currency != Item.Currency).ThrowIfTrue("存在不相符的货币单位!");
                    if (payment.Type != PaymentType.Split && payment.Type != PaymentType.Pos && payment.Type != PaymentType.Cash && payment.Type != PaymentType.AliPay && payment.Type != PaymentType.WeChat) throw new Exception("请选择正确的付款方式!");
                    if (payment.Amount - payment.AmountUsed < amountSettlement) throw new Exception($"所选付款方式余额不足,可用余额:{payment.Amount - payment.AmountUsed}");
                    (payment.AmountInvoice != payment.Amount).ThrowIfTrue("所选到账尚未开票,请先开票后再分摊!");

                    var dynamicSplitPayment = Item.Paying(payment, accountBalance, accountBiz);//insert
                    var accountDetails = dynamicSplitPayment.Using(accountBalance, accountBiz);//insert
                    var payments = new List<Payment>();
                    payments.Add(dynamicSplitPayment);
                    var payPayments = payments.Correlating(settlements);//建立Payment和Settlement关系,更新Settlement的AmountPayment值

                    var trans = conn.OpenTransaction();
                    List<IDbTransaction> subTrans = new List<IDbTransaction>();
                    try
                    {
                        conn.Insert(Item);//创建付款单
                        rtns.Results.Add(Item);
                        dynamicSplitPayment.SerialNumber = payment.GetSubSerialNumber(conn);
                        conn.Insert(dynamicSplitPayment);//创建DynamicSplitPayment
                        accountDetails.ForEach(o => o.Insert(conn));//创建账户明细
                        payment.RecalculateAmountUsed(conn);//重新统计到账的使用情况
                        foreach (var item in payPayments)
                        {
                            item.SerialNumber = payment.GetSubSerialNumber(conn);
                            conn.Insert(item);
                        }

                        #region 同步更新泛型Settlement表中数据
                        foreach (var item in settlements)
                        {
                            conn.UpdateOnly(item);//更新了Paykvid和AmountPayment
                        }
                        var groupSettlement = settlements.GroupBy(o => o.BizType);
                        if (_realMethod == null)
                        {
                            var function = new Func<IDbConnection, List<Settlement>, object>(onExecution<Entity>);
                            _realMethod = this.GetType().GetMethod(function.Method.Name, BindingFlags.NonPublic | BindingFlags.Instance);
                        }
                        foreach (var group in groupSettlement)
                        {
                            var entityType = Text.AssemblyUtils.FindType(group.Key);
                            if (entityType == null) continue;
                            if (entityType == typeof(Settlement)) continue;
                            var result = _realMethod.MakeGenericMethod(entityType).Invoke(this, new object[2] { conn, group.ToList() });
                            if (result != null) subTrans.Add(result as IDbTransaction);
                        }
                        #endregion

                        trans.Commit();
                        subTrans.ForEach(o => o?.Commit());
                    }
                    catch (Exception ex)
                    {
                        trans.Rollback();
                        subTrans.ForEach(o => o?.Rollback());
                        throw ex;
                    }
                }
                if (payingMethod.Type == PayType.Account)
                {
                    var account = conn.SingleById<Account>(payingMethod.Kvid);
                    account.ThrowIfNull("未找到所选账户!");
                    (account.Currency != Item.Currency).ThrowIfTrue("存在不相符的货币单位!");
                    (account.Type != AccountType.Deposit).ThrowIfTrue("请选择正确的客户存款账户!");
                    (account.Amount + account.CreditLine < amountSettlement).ThrowIfTrue($"所选付款方式余额不足,可用余额:{account.Amount + account.CreditLine}");

                    var dynamicSplitPayment = Item.Paying(account, accountBiz);//insert
                    var accountDetails = dynamicSplitPayment.Using(account, accountBiz);//insert
                    var payments = new List<Payment>();
                    payments.Add(dynamicSplitPayment);
                    var payPayments = payments.Correlating(settlements);//建立Payment和Settlement关系,更新Settlement的AmountPayment值

                    var trans = conn.OpenTransaction();
                    List<IDbTransaction> subTrans = new List<IDbTransaction>();
                    try
                    {
                        conn.Insert(Item);//创建付款单
                        rtns.Results.Add(Item);
                        conn.Insert(dynamicSplitPayment);//创建DynamicSplitPayment
                        accountDetails.ForEach(o => o.Insert(conn));//创建账户明细
                        foreach (var item in payPayments)
                        {
                            item.SerialNumber = dynamicSplitPayment.GetSubSerialNumber(conn);
                            conn.Insert(item);
                        }

                        #region 同步更新泛型Settlement表中数据
                        foreach (var item in settlements)
                        {
                            conn.UpdateOnly(item);//更新了Paykvid和AmountPayment
                        }
                        var groupSettlement = settlements.GroupBy(o => o.BizType);
                        if (_realMethod == null)
                        {
                            var function = new Func<IDbConnection, List<Settlement>, object>(onExecution<Entity>);
                            _realMethod = this.GetType().GetMethod(function.Method.Name, BindingFlags.NonPublic | BindingFlags.Instance);
                        }
                        foreach (var group in groupSettlement)
                        {
                            var entityType = Text.AssemblyUtils.FindType(group.Key);
                            if (entityType == null) continue;
                            if (entityType == typeof(Settlement)) continue;
                            var result = _realMethod.MakeGenericMethod(entityType).Invoke(this, new object[2] { conn, group.ToList() });
                            if (result != null) subTrans.Add(result as IDbTransaction);
                        }
                        #endregion

                        trans.Commit();
                        subTrans.ForEach(o => o?.Commit());
                    }
                    catch (Exception ex)
                    {
                        trans.Rollback();
                        subTrans.ForEach(o => o?.Rollback());
                        throw ex;
                    }
                }
                if (payingMethod.Type == PayType.Discount)
                {
                    var discountAccount = conn.SingleById<Account>(payingMethod.Kvid);
                    discountAccount.ThrowIfNull("未找到所选账户!");
                    (discountAccount.Currency != Item.Currency).ThrowIfTrue("存在不相符的货币单位!");
                    (discountAccount.Type != AccountType.Discount).ThrowIfTrue("请选择正确的折扣账户!");
                    (discountAccount.OwnerKvid != KiviiContext.CurrentMember.DepartmentKvid).ThrowIfTrue("存在非本部门的折扣账户!");

                    var dynamicSplitPayment = Item.Paying(discountAccount, accountBiz);//insert
                    var discount = Item.Discount(discountAccount, amountSettlement, payingMethod.Remark);
                    var accountDetails = dynamicSplitPayment.Using(discountAccount, accountBiz);//insert
                    var payments = new List<Payment>();
                    payments.Add(dynamicSplitPayment);
                    var payPayments = payments.Correlating(settlements);//建立Payment和Settlement关系,更新Settlement的AmountPayment值

                    var trans = conn.OpenTransaction();
                    List<IDbTransaction> subTrans = new List<IDbTransaction>();
                    try
                    {
                        conn.Insert(Item);//创建付款单
                        conn.Insert(discount);//创建折扣日志
                        rtns.Results.Add(Item);
                        conn.Insert(dynamicSplitPayment);//创建DynamicSplitPayment
                        accountDetails.ForEach(o => o.Insert(conn));//创建账户明细
                        foreach (var item in payPayments)
                        {
                            item.SerialNumber = dynamicSplitPayment.GetSubSerialNumber(conn);
                            conn.Insert(item);
                        }

                        #region 同步更新泛型Settlement表中数据
                        foreach (var item in settlements)
                        {
                            conn.UpdateOnly(item);//更新了Paykvid和AmountPayment
                        }
                        var groupSettlement = settlements.GroupBy(o => o.BizType);
                        if (_realMethod == null)
                        {
                            var function = new Func<IDbConnection, List<Settlement>, object>(onExecution<Entity>);
                            _realMethod = this.GetType().GetMethod(function.Method.Name, BindingFlags.NonPublic | BindingFlags.Instance);
                        }
                        foreach (var group in groupSettlement)
                        {
                            var entityType = Text.AssemblyUtils.FindType(group.Key);
                            if (entityType == null) continue;
                            if (entityType == typeof(Settlement)) continue;
                            var result = _realMethod.MakeGenericMethod(entityType).Invoke(this, new object[2] { conn, group.ToList() });
                            if (result != null) subTrans.Add(result as IDbTransaction);
                        }
                        #endregion

                        trans.Commit();
                        subTrans.ForEach(o => o?.Commit());
                    }
                    catch (Exception ex)
                    {
                        trans.Rollback();
                        subTrans.ForEach(o => o?.Rollback());
                        throw ex;
                    }
                }
                return rtns;
            }
            else//组合付款登记
            {
                var paymentKvids = PayingMethods.Where(o => o.Type == PayType.Payment).ToList();
                var accountKvids = PayingMethods.Where(o => o.Type == PayType.Account).ToList();
                var discountKvids = PayingMethods.Where(o => o.Type == PayType.Discount).ToList();
                List<Payment> payments = null;
                if (!paymentKvids.IsNullOrEmpty())
                {
                    payments = conn.SelectByIds<Payment>(paymentKvids);
                    payments.ThrowIfNullOrEmpty("未找到所选付款数据!");
                    payments.Exists(o => o.Currency != Item.Currency).ThrowIfTrue("存在不相符的货币单位!");
                    if (payments.Exists(o => o.Type != PaymentType.Split && o.Type != PaymentType.Pos && o.Type != PaymentType.Cash && o.Type != PaymentType.AliPay && o.Type != PaymentType.WeChat)) throw new Exception("请选择正确的付款方式!");
                    if (payments.Exists(o => (o.Amount - o.AmountUsed) <= 0)) throw new Exception($"所选付款方式余额不足!");
                    if (payments.Exists(o => (o.AmountInvoice != o.Amount))) throw new Exception($"所选付款方式余额不足!");

                }
                List<Account> accounts = null;
                if (!accountKvids.IsNullOrEmpty())
                {
                    accounts = conn.SelectByIds<Account>(accountKvids);
                    accounts.ThrowIfNullOrEmpty("未找到所选账户!");
                    accounts.Exists(o => o.Currency != Item.Currency).ThrowIfTrue("存在不相符的货币单位!");
                    if (accounts.Exists(o => o.Type != AccountType.Deposit)) throw new Exception("请选择正确的客户存款账户!");
                    if (accounts.Exists(o => (o.Amount + o.CreditLine) <= 0)) throw new Exception($"所选付款方式余额不足!");

                }
                List<Account> discounts = null;
                if (!discountKvids.IsNullOrEmpty())
                {
                    discounts = conn.SelectByIds<Account>(discountKvids);
                    discounts.ThrowIfNullOrEmpty("未找到所选账户!");
                    discounts.Exists(o => o.Currency != Item.Currency).ThrowIfTrue("存在不相符的货币单位!");
                    if (discounts.Exists(o => o.Type != AccountType.Discount)) throw new Exception("请选择正确的折扣账户!");
                    if (discounts.Exists(o => o.OwnerKvid != KiviiContext.CurrentMember.DepartmentKvid)) throw new Exception($"存在非本部门的折扣账户!");

                }

                var dynamicSplitPayments = new List<Payment>();
                var allAccountDetails = new List<AccountDetail>();
                var allDiscounts = new List<Discount>();

                if (!payments.IsNullOrEmpty())
                {
                    foreach (var item in payments)
                    {
                        var payingMethod = PayingMethods.FirstOrDefault(o => o.Kvid == item.Kvid);
                        if (payingMethod == null) continue;
                        var dynamicSplitPayment = Item.Paying(item, accountBalance, accountBiz, payingMethod.Amount);//insert
                        var accountDetails = dynamicSplitPayment.Using(accountBalance, accountBiz);//insert
                        dynamicSplitPayments.Add(dynamicSplitPayment);
                        allAccountDetails.AddRange(accountDetails);
                    }
                }
                if (!accounts.IsNullOrEmpty())
                {
                    foreach (var item in accounts)
                    {
                        var payingMethod = PayingMethods.FirstOrDefault(o => o.Kvid == item.Kvid);
                        if (payingMethod == null) continue;
                        var dynamicSplitPayment = Item.Paying(item, accountBiz, payingMethod.Amount);//insert
                        var accountDetails = dynamicSplitPayment.Using(item, accountBiz);//insert
                        dynamicSplitPayments.Add(dynamicSplitPayment);
                        allAccountDetails.AddRange(accountDetails);
                    }
                }
                if (!discounts.IsNullOrEmpty())
                {
                    foreach (var item in discounts)
                    {
                        var payingMethod = PayingMethods.FirstOrDefault(o => o.Kvid == item.Kvid);
                        if (payingMethod == null) continue;
                        var dynamicSplitPayment = Item.Paying(item, accountBiz, payingMethod.Amount);//insert
                        var discount = Item.Discount(item, amountSettlement, payingMethod.Remark);
                        var accountDetails = dynamicSplitPayment.Using(item, accountBiz);//insert
                        allDiscounts.Add(discount);
                        dynamicSplitPayments.Add(dynamicSplitPayment);
                        allAccountDetails.AddRange(accountDetails);
                    }
                }

                var payPayments = dynamicSplitPayments.Correlating(settlements);//建立Payment和Settlement关系,更新Settlement的AmountPayment值

                var trans = conn.OpenTransaction();
                List<IDbTransaction> subTrans = new List<IDbTransaction>();
                try
                {
                    conn.Insert(Item);//创建付款单
                    rtns.Results.Add(Item);
                    foreach (var item in allDiscounts)
                    {
                        conn.Insert(item);//创建折扣日志
                    }
                    foreach (var item in dynamicSplitPayments)
                    {
                        item.SerialNumber = item.GetSubSerialNumber(conn);
                        conn.Insert(item);//创建DynamicSplitPayment
                        if (item.RootKvid != item.Kvid) item.RecalculateAmountUsed(conn);//重新统计到账的使用情况
                    }
                    allAccountDetails.ForEach(o => o.Insert(conn));//创建账户明细
                    foreach (var item in payPayments)
                    {
                        item.SerialNumber = item.GetSubSerialNumber(conn);
                        conn.Insert(item);
                    }

                    #region 同步更新泛型Settlement表中数据
                    foreach (var item in settlements)
                    {
                        conn.UpdateOnly(item);//更新了Paykvid和AmountPayment
                    }
                    var groupSettlement = settlements.GroupBy(o => o.BizType);
                    if (_realMethod == null)
                    {
                        var function = new Func<IDbConnection, List<Settlement>, object>(onExecution<Entity>);
                        _realMethod = this.GetType().GetMethod(function.Method.Name, BindingFlags.NonPublic | BindingFlags.Instance);
                    }
                    foreach (var group in groupSettlement)
                    {
                        var entityType = Text.AssemblyUtils.FindType(group.Key);
                        if (entityType == null) continue;
                        if (entityType == typeof(Settlement)) continue;
                        var result = _realMethod.MakeGenericMethod(entityType).Invoke(this, new object[2] { conn, group.ToList() });
                        if (result != null) subTrans.Add(result as IDbTransaction);
                    }
                    #endregion

                    trans.Commit();
                    subTrans.ForEach(o => o?.Commit());
                }
                catch (Exception ex)
                {
                    trans.Rollback();
                    subTrans.ForEach(o => o?.Rollback());
                    throw ex;
                }

                return rtns;
            }
        }

        private IDbTransaction onExecution<T>(IDbConnection conn, List<Settlement> settlements) where T : IEntity
        {
            var connE = KiviiContext.GetOpenedDbConnection<EntitySettlement<T>>();
            #region  合并数据库连接,并启用事务
            IDbTransaction tranE = null;
            if (connE.ConnectionString == conn.ConnectionString)
            {
                connE.Close();
                connE = conn;
            }
            else
            {
                tranE = connE.OpenTransaction();
            }
            #endregion
            var instance = Activator.CreateInstance(typeof(T)) as IEntityHasPayment;
            foreach (var item in settlements)
            {
                try
                {
                    var settlement = new EntitySettlement<T>();
                    settlement.PopulateInstance(item);
                    settlement.AddOnlyProperties(o => o.PayKvid);
                    settlement.AddOnlyProperties(o => o.AmountPayment);
                    connE.UpdateOnly(settlement);

                    if (instance != null)
                    {
                        instance.Kvid = settlement.BizKvid;
                        instance.AmountPayment = settlement.AmountPayment;
                        instance.AddOnlyProperties(o => o.AmountPayment);
                        connE.UpdateOnly(instance);
                    }
                }
                catch
                {
                    continue;
                }
            }
            return tranE;
        }
    }

    [Api(Description = "作废登记")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PayOffset : RestfulExecution<Pay>
    {
        public Guid Kvid { get; set; }
        public string Remark { get; set; }

        private static MethodInfo _realMethod = null;

        public override object OnExecution(IRequest req, IResponse res)
        {
            Kvid.ThrowIfEmpty("未传入要冲账的付款登记!");

            var conn = KiviiContext.GetOpenedDbConnection<Pay>();
            var pay = conn.SingleById<Pay>(Kvid);
            pay.ThrowIfNull("未找到对应数据!");
            if (pay.OffsetKvid != Guid.Empty) throw new Exception("已冲账,请勿重复重复操作!");
            if (pay.BizKvid != Guid.Empty) throw new Exception("已关联业务,请先解除关联!");

            var settlements = conn.Select<Settlement>(o => o.OffsetKvid == Guid.Empty && o.PayKvid == Kvid);//付款登记的结算
            var dynamicPayments = conn.Select<Payment>(o => o.OffsetKvid == Guid.Empty && o.BizKvid == Kvid);//使用的付款信息 要作废
            List<Payment> offsetPayments = null;//要作废的关联数据
            List<Payment> payPayments = null;//要作废的关联数据
            List<AccountDetail> allAccountDetails = null;//退回对应账户生成的明细
            var offsetPay = pay.Offset($"付款登记作废,:{Remark}");

            Account payerAccount = null;
            Account payeeAccount = null;
            if (!dynamicPayments.IsNullOrEmpty())
            {
                dynamicPayments.Exists(o => o.Type != PaymentType.DynamicSplit).ThrowIfTrue("存在不正确的付款数据类型!");
                dynamicPayments.Exists(o => o.PayerAccountKvid == Guid.Empty || o.PayeeAccountKvid == Guid.Empty).ThrowIfTrue("存在无收付账户信息的记录!");
                payPayments = conn.Select<Payment>(o => o.OffsetKvid == Guid.Empty && Sql.In(o.ParentKvid, dynamicPayments.ConvertAll(p => p.Kvid)));
                foreach (var dynamicPayment in dynamicPayments)
                {
                    if (payerAccount == null || payerAccount.Kvid != dynamicPayment.PayerAccountKvid)
                    {
                        payerAccount = conn.SingleById<Account>(dynamicPayment.PayerAccountKvid);
                        payerAccount.ThrowIfNull("未找到当前对应账户");
                    }
                    if (payeeAccount == null || payeeAccount.Kvid != dynamicPayment.PayeeAccountKvid)
                    {
                        payeeAccount = conn.SingleById<Account>(dynamicPayment.PayeeAccountKvid);
                        payeeAccount.ThrowIfNull("未找到当前对应账户");
                    }

                    var accountDetails = dynamicPayment.Using(payeeAccount, payerAccount, $"付款登记作废,:{Remark}");//原路退回对应账户
                    if (allAccountDetails.IsNullOrEmpty()) allAccountDetails = new List<AccountDetail>();
                    allAccountDetails.AddRange(accountDetails);

                    var offsetPayment = dynamicPayment.Offset($"付款登记作废:{Remark}");
                    if (offsetPayments.IsNullOrEmpty()) offsetPayments = new List<Payment>();
                    offsetPayments.Add(offsetPayment);
                }
            }
            if (!payPayments.IsNullOrEmpty())
            {
                foreach (var item in payPayments)
                {
                    var offsetPayment = item.Offset($"付款登记作废:{Remark}");
                    if (offsetPayments.IsNullOrEmpty()) offsetPayments = new List<Payment>();
                    offsetPayments.Add(offsetPayment);
                }
            }

            var rtns = new RestfulExecutionResponse<Pay>();
            rtns.Results = new List<Pay>();

            var trans = conn.OpenTransaction();
            List<IDbTransaction> subTrans = new List<IDbTransaction>();
            try
            {
                if (!payPayments.IsNullOrEmpty()) payPayments.ForEach(o => conn.UpdateOnly(o));
                if (!offsetPayments.IsNullOrEmpty()) offsetPayments.ForEach(o => conn.Insert(o));
                if (!dynamicPayments.IsNullOrEmpty())
                {
                    dynamicPayments.ForEach(o => conn.UpdateOnly(o));
                    foreach (var item in dynamicPayments)
                    {
                        if (item.RootKvid != item.Kvid) item.RecalculateAmountUsed(conn);
                    }
                }
                if (!allAccountDetails.IsNullOrEmpty()) allAccountDetails.ForEach(o => o.Insert(conn));

                if (!settlements.IsNullOrEmpty())
                {
                    var updateLamda = conn.From<Settlement>();
                    updateLamda = updateLamda.Update(o => new { o.PayKvid, o.AmountPayment });
                    updateLamda = updateLamda.Where(o => Sql.In(o.Kvid, settlements.ConvertAll(p => p.Kvid)));
                    conn.UpdateOnly<Settlement>(new Settlement { PayKvid = Guid.Empty, AmountPayment = 0 }, updateLamda);

                    var groupSettlement = settlements.GroupBy(o => o.BizType);
                    if (_realMethod == null)
                    {
                        var function = new Func<IDbConnection, List<Settlement>, object>(onExecution<Entity>);
                        _realMethod = this.GetType().GetMethod(function.Method.Name, BindingFlags.NonPublic | BindingFlags.Instance);
                    }
                    foreach (var group in groupSettlement)
                    {
                        var entityType = Text.AssemblyUtils.FindType(group.Key);
                        if (entityType == null) continue;
                        if (entityType == typeof(Settlement)) continue;
                        var result = _realMethod.MakeGenericMethod(entityType).Invoke(this, new object[2] { conn, group.ToList() });
                        if (result != null) subTrans.Add(result as IDbTransaction);
                    }
                }

                conn.UpdateOnly(pay);
                rtns.Results.Add(pay);
                conn.Insert(offsetPay);
                trans.Commit();
                subTrans.ForEach(o => o?.Commit());
            }
            catch (Exception ex)
            {

                trans.Rollback();
                subTrans.ForEach(o => o?.Rollback());
                throw ex;
            }

            return rtns;
        }

        private IDbTransaction onExecution<T>(IDbConnection conn, List<Settlement> settlements) where T : IEntity
        {
            var connE = KiviiContext.GetOpenedDbConnection<EntitySettlement<T>>();
            #region  合并数据库连接,并启用事务
            IDbTransaction tranE = null;
            if (connE.ConnectionString == conn.ConnectionString)
            {
                connE.Close();
                connE = conn;
            }
            else
            {
                tranE = connE.OpenTransaction();
            }
            #endregion
            var updateLamda = connE.From<EntitySettlement<T>>();
            updateLamda = updateLamda.Update(o => new { o.PayKvid, o.AmountPayment });
            updateLamda = updateLamda.Where(o => Sql.In(o.Kvid, settlements.ConvertAll(p => p.Kvid)));
            connE.UpdateOnly<EntitySettlement<T>>(new EntitySettlement<T> { PayKvid = Guid.Empty, AmountPayment = 0 }, updateLamda);

            var instance = Activator.CreateInstance(typeof(T)) as IEntityHasPayment;
            foreach (var item in settlements)
            {
                try
                {
                    if (instance != null)
                    {
                        instance.Kvid = item.BizKvid;
                        instance.AmountPayment = 0;
                        instance.AddOnlyProperties(o => o.AmountPayment);
                        connE.UpdateOnly(instance);
                    }
                }
                catch
                {
                    continue;
                }
            }
            return tranE;
        }
    }

    [Api(Description = "更新")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PayUpdate : RestfulExecution<Pay>
    {
        public Pay Item { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            (Item == null).ThrowIfTrue("参数错误:未传入付款单信息!");
            (Item.Kvid == Guid.Empty).ThrowIfTrue("参数错误:未传入付款单Kvid!");

            var conn = KiviiContext.GetOpenedDbConnection<Pay>();
            var pay = conn.SingleById<Pay>(Item.Kvid);
            (pay == null).ThrowIfTrue("未找到相应付款单信息!");
            if (Item.OnlyPropertiesIsExist(o => o.Summary))
            {
                pay.Summary = Item.Summary;
                pay.AddOnlyProperties(o => o.Summary);
            }
            if (Item.OnlyPropertiesIsExist(o => o.Remark))
            {
                pay.Remark = Item.Remark;
                pay.AddOnlyProperties(o => o.Remark);
            }

            var rtns = new RestfulUpdateResponse<Pay>();
            rtns.Results = new List<Pay>();
            if (pay.OnlyProperties.Count > 0)
            {
                conn.UpdateOnly(pay);
                rtns.Results.Add(pay);
            }
            return rtns;
        }
    }

    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PayRead : RestfulRead<Pay> { }

    [Api(Description = "查询")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PayQuery : RestfulExecution<Pay>
    {
        #region QueryArgs
        public virtual int? Skip { get; set; }

        public virtual int? Take { get; set; }

        public virtual string OrderBy { get; set; }

        public string OrderByDesc { get; set; }

        public virtual string Include { get; set; }

        public virtual string Fields { get; set; }

        public string QueryKeys { get; set; }

        public string QueryValues { get; set; }

        #endregion

        public override object OnExecution(IRequest req, IResponse res)
        {
            var conn = KiviiContext.GetOpenedDbConnection<Pay>();
            var dynamicParams = Request.GetRequestParams();
            var autoQuery = Request.TryResolve<IAutoQueryDb>();
            autoQuery.IncludeTotal = true;
            var request = new RestfulQuery<Pay>();
            request = request.PopulateWith(this);
            var sqlExpress = autoQuery.CreateQuery(Request, conn, request, dynamicParams);
            var rtns = autoQuery.Execute(Request, conn, request, sqlExpress);
            return rtns;
        }
    }

}
